<?php

namespace Repository;

use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;
use Zeedhi\DevOps\Util\Utilities;

class CpCall
{
    use TQuery;

    public function getEntityManager(){
        return InstanceProvider::getEntityManager();
    }

    public function findAllCalls($customQuery, $orderBy, $page, $pageSize)
    {
        $startPos = ($page - 1) * $pageSize;
        $finalPos = $page * $pageSize;
        $query = <<<SQL
            SELECT   COUNT(1) OVER () TOTAL_LINES_ZHNEXT,CALL.NMOPERRESP, CALL.ID, CALL.NRORG, CALL.CDCLIENTE, CALL.CDFILIAL, CALL.CALL_RELATIONSHIP, CALL.TEKNISA_ID,
                   CALL.REFERENCE, CALL.CLASSIFICATION, CALL.CLASSIFICATION_NAME, CALL.REPORTED_BY, CALL.IS_ACTIVE,
                   CALL.REPORTER_NAME, CALL.PRODUCT, CALL.CAUSE, CALL.CAUSE_NAME, CALL.PRODUCT_NAME, CALL.CREATED_ORDER,
                   CALL.UPDATED, TO_CHAR(CALL.CREATED_AT, 'DD/MM/YYYY') AS CREATED_AT, CALL.CLOSED, CALL.STATUS,
                   CALL.PLANNED_DATE, CALL.UPDATED_ORDER, TO_CHAR(CALL.UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT,
                   CALL.MAIN_TYPE_CALL, CALL.MAIN_TYPE_CALL_DESCRIPTION, CALL.TYPE, CALL.VERSION_DEV,
                   (SELECT RTRIM(XMLAGG(XMLELEMENT(E, LABEL.DESCRIPTION, ',')
                                          .EXTRACT('//text()')
                                          ORDER BY LABEL.DESCRIPTION
                                       ).GETCLOBVAL(), ','
                                ) AS LABEL_DESCRIPTION
                      FROM CP_CALL_LABEL_RELATION LABEL_REL
                      JOIN CP_CALL_LABEL LABEL ON LABEL_REL.LABEL = LABEL.NRCALLLABEL
                     WHERE LABEL_REL.CALL = CALL.ID
                       ) AS LABEL_DESCRIPTION,
                   CALL.NMRAZSOCCLIE, CALL.NMFANTCLIE, CALL.TEKNISA_INTERLOCUTOR_CODE, CALL.PLANNED_VERSION,
                   CALL.PLANNED_VALUE, CALL.RESPONSABLE_DIVISION_CODE, CALL.CUSTOMER_INTERLOCUTOR_CODE,
                   CALL.CUSTOMER_RESPONSABLE_CODE, CALL.CUSTOMER_INTERLOCUTOR, CALL.CALL_PRIORITY, CALL.RESPONSIBLE_OF_THE_PRODUCT,
                   CALL.RESPONSIBLE_OF_THE_CUSTOMER, CALL.TEKNISA_INTERLOCUTOR, CALL.RESPONSABLE_DIVISION,
                   CALL.TEXT_SMALL_1, CALL.TEXT_SMALL_2, CALL.TEXT_SMALL_3, CALL.TEXT_SMALL_4, CALL.TEXT_SMALL_5,
                   CALL.TEXT_LONG_1,CALL.TEXT_LONG_2,CALL.TEXT_LONG_3,CALL.TEXT_LONG_4,CALL.TEXT_LONG_5,
                   CALL.NUMBER_1,CALL.NUMBER_2,CALL.NUMBER_3,CALL.NUMBER_4,CALL.NUMBER_5,CALL.
                   DATE_1,CALL.DATE_2,CALL.DATE_3,CALL.DATE_4,CALL.DATE_5,CALL.PRODUCT_TYPE_NAME
              FROM ($customQuery) CALL
         LEFT JOIN CP_CALL_LABEL_RELATION LABEL_REL ON LABEL_REL.CALL = CALL.ID
         LEFT JOIN CP_CALL_LABEL LABEL ON LABEL_REL.LABEL = LABEL.NRCALLLABEL
          GROUP BY CALL.NMOPERRESP, CALL.ID, CALL.NRORG, CALL.CDCLIENTE, CALL.CDFILIAL, CALL.CALL_RELATIONSHIP, CALL.TEKNISA_ID,
                   CALL.REFERENCE, CALL.CLASSIFICATION, CALL.CLASSIFICATION_NAME, CALL.REPORTED_BY, CALL.IS_ACTIVE,
                   CALL.REPORTER_NAME, CALL.PRODUCT, CALL.CAUSE, CALL.CAUSE_NAME, CALL.PRODUCT_NAME, CALL.CREATED_ORDER,
                   CALL.UPDATED, CALL.CREATED_AT, CALL.CLOSED, CALL.STATUS, CALL.PLANNED_DATE, CALL.UPDATED_ORDER,
                   CALL.UPDATED_AT, CALL.MAIN_TYPE_CALL, CALL.MAIN_TYPE_CALL_DESCRIPTION, CALL.TYPE, CALL.NMRAZSOCCLIE,
                   CALL.NMFANTCLIE, CALL.TEKNISA_INTERLOCUTOR_CODE, CALL.PLANNED_VERSION, CALL.PLANNED_VALUE,
                   CALL.RESPONSABLE_DIVISION_CODE, CALL.CUSTOMER_INTERLOCUTOR_CODE, CALL.CUSTOMER_RESPONSABLE_CODE,
                   CALL.CUSTOMER_INTERLOCUTOR, CALL.CALL_PRIORITY, CALL.RESPONSIBLE_OF_THE_PRODUCT,
                   CALL.RESPONSIBLE_OF_THE_CUSTOMER, CALL.TEKNISA_INTERLOCUTOR, CALL.RESPONSABLE_DIVISION, CALL.VERSION_DEV,
                   CALL.TEXT_SMALL_1, CALL.TEXT_SMALL_2, CALL.TEXT_SMALL_3, CALL.TEXT_SMALL_4, CALL.TEXT_SMALL_5,
                   CALL.TEXT_LONG_1,CALL.TEXT_LONG_2,CALL.TEXT_LONG_3,CALL.TEXT_LONG_4,CALL.TEXT_LONG_5,
                   CALL.NUMBER_1,CALL.NUMBER_2,CALL.NUMBER_3,CALL.NUMBER_4,CALL.NUMBER_5,CALL.DATE_1,
                   CALL.DATE_2,CALL.DATE_3,CALL.DATE_4,CALL.DATE_5,CALL.PRODUCT_TYPE_NAME
                   ORDER BY $orderBy
SQL;
        //TODO buga o ordenar por data
        //return $this->createAndExecuteQueryAndBuildResponseData(InstanceProvider::getEntityManager(), $query);
        $query = Utilities::getPaginatedQuery($query,$finalPos,$startPos);
        $result =  $this->getEntityManager()->getConnection()->fetchAll($query);
        $total = empty($result) ? 0 : $result[0]['TOTAL_LINES_ZHNEXT'];
        $returnNext = [ 'data' => $result, 'pagination' => [ 'limit' => $pageSize, 'page' => $page, 'total' => $total]];
        return $returnNext;
    }

    public function findSoliservById($soliservId, $situationFilters, $userId)
    {
        $gepOpened  = $situationFilters['GEP']['OPENED'];
        $gepWaiting = $situationFilters['GEP']['WAITING'];
        $gepClosed  = $situationFilters['GEP']['CLOSED'];
        $validate = \Util\Utilities::getValidateBudgetFaseGep();
        $refuse = \Util\Utilities::getRefuseBudgetFaseGep();

        $query = <<<SQL
        SELECT  SER.NRSOLISERV AS ID, SER.NRORG, SER.DSREFESOLI AS REFERENCE,
                SER.CDOPERINCLUSAO AS CREATED_BY, SER.NRLICENCAUSO AS USE_LICENSE,
                DES.DSSOLISERV AS DESCRIPTION, SER.CDOPERINCLUSAO AS REPORTER_BY,
                TO_CHAR((SELECT MAX(LFT.DTINCLUSAO) FROM LOGFASETRAM LFT WHERE (LFT.NRSOLISERV = :SOLISERV_ID AND LFT.CDFILIAL = SER.CDFILIAL) ), 'DD/MM/YYYY') AS UPDATED_AT,
                SER.CDPRODCOMP AS PRODUCT, 'S' AS TYPE, OPE.NMOPERADOR AS REPORTER_NAME,
                '' AS CAUSE, SER.CDOPERULTATU AS UPDATED, PRO.CDPRODUTO AS BOARD,
                SER.CDPRODGRUP AS PRODUCT_TYPE, PRT.NMPRODUTO AS PRODUCT_TYPE_NAME, CLI.NMRAZSOCCLIE,
                ASS.NMASSUNTO AS CLASSIFICATION_NAME, ASS.IDORIGOBRSOL AS CLASSIFICATION_IDORIGOBRSOL,
                PRO.NMPRODUTO AS PRODUCT_NAME, ASS.CDASSUNTO AS CLASSIFICATION,
                 CASE WHEN SER.CDFASETRAM IN ('$validate')
                       THEN 'Y'
                       ELSE 'N'
                     END AS ENABLE_APPROVAL,
                 CASE WHEN EXISTS (   SELECT NRSEQORCAMENTO AS EXTERNAL_ID
                                        FROM FCT_ORCAMENTO
                                       WHERE NRSOLISERV = SER.NRSOLISERV
                                         AND IDATIVO = 'S'
                                         AND IDSITUACAOORCAMENTO <> 'CANCELADO')
                       THEN 'Y'
                       ELSE 'N'
                 END AS IS_INTEGRATION_SM,
                CASE WHEN $gepWaiting AND SER.CDFASETRAM NOT IN (SELECT CDFATRAMENCE FROM PARAMGERAL PARA WHERE PARA.NRORG = SER.NRORG)
                     THEN 'Y'
                     ELSE 'N'
                 END AS ENABLE_CLOSED,
                CASE WHEN $gepClosed
                     THEN 'Fechado'
                     WHEN $gepWaiting
                     THEN 'Aguardando ação do cliente'
                     WHEN $gepOpened
                     THEN 'Em aberto'
                     ELSE 'Em progresso'
                END AS STATUS,
                CASE WHEN $gepWaiting
                     THEN 'Y'
                     ELSE 'N'
                 END AS WAITING,
                (SELECT S.NRSOLISERV
                   FROM SOLISERV S
                   JOIN ORCASOLICITA O ON (O.CDFILIAL = S.CDFILIAL AND O.NRSOLISERV = S.NRSOLISERV)
                  WHERE S.NRSOLISERV = :SOLISERV_ID
                    AND ROWNUM = 1
                    AND  (SELECT CDFASETRAM
                                   FROM ( SELECT CDFASETRAM
                                            FROM LOGFASETRAM LFT
                                           WHERE LFT.NRSOLISERV = :SOLISERV_ID
                                             AND CDFASETRAM IN ('$refuse','$validate')
                                        ORDER BY DTALFASETRAM DESC)
                                  WHERE ROWNUM = 1) IN ('$validate')) AS WITH_BUDGET,
                SER.CDVERPROORIG AS PRODUCT_VERSION, SER.CDVERPROORIG AS PRODUCT_VERSION_LABEL,
                SER.CDRELVERORIG AS PRODUCT_RELEASE, SER.CDRELVERORIG AS PRODUCT_RELEASE_LABEL,
                SER.CDPATRELORIG AS PRODUCT_PATCH, SER.CDPATRELORIG AS PRODUCT_PATCH_LABEL
         FROM  SOLISERV SER
         JOIN  DESCSOLI DES ON DES.NRSOLISERV = SER.NRSOLISERV
         JOIN  OPERADOR OPE ON OPE.CDOPERADOR = SER.CDOPERADOR
         JOIN  PRODUTO PRO  ON PRO.CDPRODUTO = SER.CDPRODCOMP
    LEFT JOIN  PRODUTO PRT  ON PRT.CDPRODUTO = SER.CDPRODGRUP
         JOIN  ASSUNTO ASS  ON ASS.CDASSUNTO = SER.CDASSUNTO
    LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SER.CDCLIENTE
        WHERE  SER.NRSOLISERV = :SOLISERV_ID
SQL;

        $params = array(
            "SOLISERV_ID" => $soliservId,
            "USER_ID" => $userId
        );
        $result = InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);

        return count($result) > 0 ? $result[0] : array();
    }

    public function findIssueById($issueId, $situationFilters = null)
    {
        $gepWaiting = '0';
        if($situationFilters){
            $gepWaiting = implode($situationFilters['PM']['WAITING'],',');
        }
        $query = <<<SQL
              SELECT  TAS.ID, TAS.ORGANIZATION AS NRORG, TAS.NAME AS REFERENCE,
                      TAS.CREATED_BY, TAS.DESCRIPTION, TAS.CREATED_BY AS REPORTER_BY,
                      TO_CHAR(TAS.UPDATED_AT, 'DD/MM/YYYY') AS UPDATED_AT,
                      TAS.PROJECT, TAS.PROJECT AS PRODUCT, RT.NAME AS REPORTER_NAME,
                      TAS.ID AS CALL_RELATIONSHIP, '' AS CAUSE, TAS.UPDATED_BY AS CDOPERULTATU, CLI.NMRAZSOCCLIE,
                      TAS.STATUS,TAS.STATUS AS STATUS,PRO.ID AS BOARD,
                      PRO.NAME AS PRODUCT_NAME, CLA.ID AS CLASSIFICATION, CLA.LABEL AS CLASSIFICATION_NAME,
                      DECODE(PVD.ID, NULL, 'Última versão', PVD.VERSION || '.' || PVD.REVISION) AS VERSION_DEV,
                      CASE WHEN TS.CONTROL = 'H'
                          THEN 'Y'
                          ELSE 'N'
                      END AS ENABLE_CLOSED,
                        CASE WHEN TS.ID IN ($gepWaiting)
                             THEN 'Y'
                             ELSE 'N'
                         END AS WAITING,
                      CASE WHEN TS.CONTROL = 'A'
                             THEN 'Y'
                             ELSE 'N'
                      END AS ENABLE_APPROVAL,
                      CASE WHEN EXISTS (SELECT EXTERNAL_ID FROM BUDGET WHERE TASK = :ISSUE_ID AND EXTERNAL_ID IS NOT NULL)
                           THEN 'Y'
                           ELSE 'N'
                      END AS IS_INTEGRATION_SM,
                      (SELECT CONTROL
                         FROM (SELECT CTY.CONTROL
                                 FROM COMMENTS COM
                                 JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                WHERE COM.TASK = TAS.ID
                             ORDER BY COM.ID DESC)
                        WHERE ROWNUM = 1 ) AS LAST_COMMENT_TYPE,
                      CASE WHEN EXISTS( SELECT ID FROM TASK_PROPERTIES TPR WHERE TPR.FINAL_TASK_STATUS = TS.ID)
                              THEN 'Y'
                              ELSE null
                      END AS CLOSED
                FROM TASK TAS
                JOIN TASK_STATUS TS ON (TS.ID = TAS.TASK_STATUS)
                JOIN PROJECT PRO ON (PRO.ID = TAS.PROJECT)
                JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION)
                JOIN RESOURCE_T RT ON RT.ID = TAS.CREATED_BY
           LEFT JOIN PRODUCT_VERSION PVD ON PVD.ID = TAS.PRODUCT_VERSION_ID_DEV
           LEFT JOIN ALLOCATION SA ON (SA.TASK = TAS.ID AND SA.TYPE = 'S')
           LEFT JOIN RESOURCE_T SRE ON (SRE.ID = SA.RESOURCE_T)
           LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SRE.EXTERNAL_CODE
               WHERE TAS.ID = :ISSUE_ID
SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );

        $result =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result)>0 ? $result[0] : array();
    }

    public function findCallTypeIssue($id, $situationFilters){


        $pmWaiting = implode($situationFilters['PM']['WAITING'],',');
        $query = <<<SQL
              SELECT  C.NRCALL AS ID, C.NRORG, C.REFERENCE, C.REPORTED_BY, C.DESCRIPTION,C.REPORTED_BY AS REPORTER_BY,
                          TO_CHAR(C.DTULTATU, 'DD/MM/YYYY') AS UPDATED_AT, C.PRODUCT, C.TYPE,OPE.NMOPERADOR AS REPORTER_NAME,
                          C.CALL_RELATIONSHIP, C.CAUSE, C.CDOPERULTATU, C.IDATIVO,C.IDATIVO AS STATUS,NVL(C.PRODUCT, PRO.ID) AS BOARD, CLI.NMRAZSOCCLIE,
                          PRO.NAME AS PRODUCT_NAME,
                          CLA.ID AS CLASSIFICATION,CLA.LABEL AS CLASSIFICATION_NAME,
                            CASE WHEN TS.CONTROL = 'H'
                                 THEN 'Y'
                                 ELSE 'N'
                             END AS ENABLE_CLOSED,
                            CASE WHEN TS.CONTROL = 'A'
                                 THEN 'Y'
                                 ELSE 'N'
                             END AS ENABLE_APPROVAL,
                            CASE WHEN EXISTS (SELECT EXTERNAL_ID FROM BUDGET WHERE TASK = TAS.ID AND EXTERNAL_ID IS NOT NULL)
                                 THEN 'Y'
                                 ELSE 'N'
                            END AS IS_INTEGRATION_SM,
                            CASE WHEN TS.ID IN ($pmWaiting) OR (  SELECT CONTROL FROM (SELECT CTY.CONTROL
                                                                    FROM COMMENTS COM
                                                                    JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                                    WHERE COM.TASK = TAS.ID
                                                                    ORDER BY COM.ID DESC)
                                                                    WHERE ROWNUM = 1 ) = 'Q'
                                 THEN 'Y'
                                 ELSE null
                             END AS WAITING,
                          (
                             SELECT rtrim(xmlagg(xmlelement(e,CLR.LABEL ,',').extract('//text()') order by CLR.LABEL ).GetClobVal(),',')
                             FROM   CP_CALL_LABEL_RELATION CLR
                             WHERE  CLR.CALL = C.NRCALL
                             AND    CLR.NRORG = C.NRORG
                          ) AS CALL_LABELS,
                         (SELECT CONTROL FROM (SELECT CTY.CONTROL
                                                FROM COMMENTS COM
                                                JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                WHERE COM.TASK = TAS.ID
                                                ORDER BY COM.ID DESC)
                                                WHERE ROWNUM = 1 ) AS LAST_COMMENT_TYPE,
                         PRV.ID AS MENU_VERSION_ID,
                         DECODE(PRV.ID,NULL,NULL,(PRV.VERSION || '.' || PRV.REVISION  || '.' || PRV.BUILD)) AS MENU_VERSION,
                         DECODE(PVD.ID, NULL, 'Última versão', PVD.VERSION || '.' || PVD.REVISION) AS VERSION_DEV,
                         OPR.LABEL AS MENU_ITEM ,OPR.ID AS MENU_ITEM_ID,
                         OPR.NREXTERNAL_PRODUCT_ID AS DEVOPS_ID,
                         NVL(PRD.ID,PDP.ID) AS DEVOPS_ID_PRODUCT,C.TEXT_SMALL_1, C.TEXT_SMALL_2, C.TEXT_SMALL_3, C.TEXT_SMALL_4, C.TEXT_SMALL_5,
                         C.TEXT_LONG_1,C.TEXT_LONG_2,C.TEXT_LONG_3,C.TEXT_LONG_4,C.TEXT_LONG_5,C.NUMBER_1,C.NUMBER_2,C.NUMBER_3,C.NUMBER_4,C.NUMBER_5,
                         TO_CHAR(DATE_1, 'DD/MM/YYYY') AS DATE_1,TO_CHAR(DATE_2, 'DD/MM/YYYY') AS DATE_2,TO_CHAR(DATE_3, 'DD/MM/YYYY') AS DATE_3,TO_CHAR(DATE_4, 'DD/MM/YYYY') AS DATE_4,TO_CHAR(DATE_5, 'DD/MM/YYYY') AS DATE_5
                FROM CP_CALL C
                JOIN OPERADOR OPE ON (OPE.CDOPERADOR = C.REPORTED_BY )
                JOIN TASK TAS ON(TAS.ID = C.CALL_RELATIONSHIP)
                JOIN TASK_STATUS TS ON (TS.ID = TAS.TASK_STATUS)
                JOIN PROJECT PRO ON(PRO.ID = NVL(C.PRODUCT, TAS.PROJECT))
           LEFT JOIN PROJECT PRC ON(PRC.ID = C.PRODUCT)
           LEFT JOIN PRODUCT PRD ON(PRD.PROJECT_ID = PRC.ID)
           LEFT JOIN PRODUCT PDP ON(PDP.PROJECT_ID = PRO.ID)
           LEFT JOIN PRODUCT_VERSION PRV ON (TAS.PRODUCT_VERSION_ID = PRV.ID)
           LEFT JOIN OM_PRODUTO_ELEMENTO OPR On (TAS.MENU_ITEM_ID = OPR.ID)
           LEFT JOIN PRODUCT_VERSION PVD ON PVD.ID = TAS.PRODUCT_VERSION_ID_DEV
                JOIN CLASSIFICATION CLA ON (CLA.ID = TAS.CLASSIFICATION)
           LEFT JOIN ALLOCATION SA ON (SA.TASK = TAS.ID AND SA.TYPE = 'S')
           LEFT JOIN RESOURCE_T SRE ON (SRE.ID = SA.RESOURCE_T)
           LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SRE.EXTERNAL_CODE
               WHERE C.NRCALL = :CALL_ID
SQL;

        $params = array(
            "CALL_ID" => $id
        );

        $result =  $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return count($result)>0 ? $result[0] : array();
    }
}
