<?php

namespace Repository;

use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;

class Soliserv
{
    use TQuery;

    public function findSoliservById($soliservId, $situationFilters, $userId)
    {
        $gepOpened  = $situationFilters['GEP']['OPENED'];
        $gepWaiting = $situationFilters['GEP']['WAITING'];
        $gepClosed  = $situationFilters['GEP']['CLOSED'];
        $validate = \Util\Utilities::getValidateBudgetFaseGep();
        $refuse = \Util\Utilities::getRefuseBudgetFaseGep();

        $query = <<<SQL
        SELECT  SER.NRSOLISERV AS ID, SER.NRORG, SER.DSREFESOLI AS REFERENCE,
                SER.CDOPERINCLUSAO AS CREATED_BY, SER.NRLICENCAUSO AS USE_LICENSE,
                DES.DSSOLISERV AS DESCRIPTION, SER.CDOPERINCLUSAO AS REPORTER_BY,
                TO_CHAR((SELECT MAX(LFT.DTINCLUSAO) FROM LOGFASETRAM LFT WHERE (LFT.NRSOLISERV = :SOLISERV_ID AND LFT.CDFILIAL = SER.CDFILIAL) ), 'DD/MM/YYYY') AS UPDATED_AT,
                SER.CDPRODCOMP AS PRODUCT, 'S' AS TYPE, OPE.NMOPERADOR AS REPORTER_NAME,
                '' AS CAUSE, SER.CDOPERULTATU AS UPDATED, PRO.CDPRODUTO AS BOARD,
                SER.CDPRODGRUP AS PRODUCT_TYPE, PRT.NMPRODUTO AS PRODUCT_TYPE_NAME, CLI.NMRAZSOCCLIE,
                ASS.NMASSUNTO AS CLASSIFICATION_NAME, ASS.IDORIGOBRSOL AS CLASSIFICATION_IDORIGOBRSOL,
                PRO.NMPRODUTO AS PRODUCT_NAME, ASS.CDASSUNTO AS CLASSIFICATION,
                 CASE WHEN SER.CDFASETRAM IN ('$validate')
                       THEN 'Y'
                       ELSE 'N'
                     END AS ENABLE_APPROVAL,
                 CASE WHEN EXISTS (   SELECT NRSEQORCAMENTO AS EXTERNAL_ID
                                        FROM FCT_ORCAMENTO
                                       WHERE NRSOLISERV = SER.NRSOLISERV
                                         AND IDATIVO = 'S'
                                         AND IDSITUACAOORCAMENTO <> 'CANCELADO')
                       THEN 'Y'
                       ELSE 'N'
                 END AS IS_INTEGRATION_SM,
                CASE WHEN $gepWaiting AND SER.CDFASETRAM NOT IN (SELECT CDFATRAMENCE FROM PARAMGERAL PARA WHERE PARA.NRORG = SER.NRORG)
                     THEN 'Y'
                     ELSE 'N'
                 END AS ENABLE_CLOSED,
                CASE WHEN $gepClosed
                     THEN 'Fechado'
                     WHEN $gepWaiting
                     THEN 'Aguardando ação do cliente'
                     WHEN $gepOpened
                     THEN 'Em aberto'
                     ELSE 'Em progresso'
                END AS STATUS,
                CASE WHEN $gepWaiting
                     THEN 'Y'
                     ELSE 'N'
                 END AS WAITING,
                (SELECT S.NRSOLISERV
                   FROM SOLISERV S
                   JOIN ORCASOLICITA O ON (O.CDFILIAL = S.CDFILIAL AND O.NRSOLISERV = S.NRSOLISERV)
                  WHERE S.NRSOLISERV = :SOLISERV_ID
                    AND ROWNUM = 1
                    AND  (SELECT CDFASETRAM
                                   FROM ( SELECT CDFASETRAM
                                            FROM LOGFASETRAM LFT
                                           WHERE LFT.NRSOLISERV = :SOLISERV_ID
                                             AND CDFASETRAM IN ('$refuse','$validate')
                                        ORDER BY DTALFASETRAM DESC)
                                  WHERE ROWNUM = 1) IN ('$validate')) AS WITH_BUDGET,
                SER.CDVERPROORIG AS PRODUCT_VERSION, SER.CDVERPROORIG AS PRODUCT_VERSION_LABEL,
                SER.CDRELVERORIG AS PRODUCT_RELEASE, SER.CDRELVERORIG AS PRODUCT_RELEASE_LABEL,
                SER.CDPATRELORIG AS PRODUCT_PATCH, SER.CDPATRELORIG AS PRODUCT_PATCH_LABEL
         FROM  SOLISERV SER
         JOIN  DESCSOLI DES ON DES.NRSOLISERV = SER.NRSOLISERV
         JOIN  OPERADOR OPE ON OPE.CDOPERADOR = SER.CDOPERADOR
         JOIN  PRODUTO PRO  ON PRO.CDPRODUTO = SER.CDPRODCOMP
    LEFT JOIN  PRODUTO PRT  ON PRT.CDPRODUTO = SER.CDPRODGRUP
         JOIN  ASSUNTO ASS  ON ASS.CDASSUNTO = SER.CDASSUNTO
    LEFT JOIN CLIENTE CLI ON CLI.CDCLIENTE = SER.CDCLIENTE
        WHERE  SER.NRSOLISERV = :SOLISERV_ID
SQL;

        $params = array(
            "SOLISERV_ID" => $soliservId,
            "USER_ID" => $userId
        );
        $result = InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);

        return count($result) > 0 ? $result[0] : array();
    }
}
