<?php

namespace Service;

use Util\InstanceProvider;
use Util\Utilities;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Teknisa\Model\Teknisa\Entities;

class Call
{
    const TYPE_ISSUE        = Entities\CpCall::TYPE_ISSUE;
    const TYPE_RICH_CLIENT  = Entities\CpCall::TYPE_RICH_CLIENT;


    const MAIN_TYPE_OF_PRODUCT_CALL = "P";
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * @var \Teknisa\Model\Teknisa\Repositories\CpCall
     */
    private $callRepository;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager  = $entityManager;
        $this->environment    = InstanceProvider::getEnvironment();
        $this->callRepository = InstanceProvider::getRepository()->getCpCall();
    }

    public function findCalls($clientes, $conditions, $order = array('UPDATED_AT.desc'), $page = 1, $limit = 50): array
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $userId = $this->environment->getCurrentUserId();
        $cdProd = null;
        $filterSLA = $filterSLAClas = $filterSLAAss = '';
        //if(!empty($SLA)){
            //$filterSLA = $this->getFilterSLA($SLA);
        //}
        //if(!empty($SLAclas)){
            //list($filterSLAClas,$filterSLAAss) = $this->getFilterSLAclas($SLAclas);
        //}

        $clientePmFilter       = DevOpsUtilities::fixBugWhereINQuery($clientes, "SRE.EXTERNAL_CODE", false, true);
        $clienteIntranetFilter = DevOpsUtilities::fixBugWhereINQuery($clientes, "SER.CDCLIENTE", false, true);
        $clienteCRMFilter      = DevOpsUtilities::fixBugWhereINQuery($clientes, "CLI.CDCLIENTE", false, true);

        $situationFilters = $this->getAllSituationDividedByType();

        $orderBy = '';
        foreach ($order as $item) {
            $orderBy .=' CALL.'. str_replace("."," ",$item);
        }

        $customQuery = $this->getQueryAllCall($conditions, $situationFilters, $clientePmFilter,
             $clienteIntranetFilter, $clienteCRMFilter, $filterSLA, $filterSLAClas, $filterSLAAss, $cdProd);

        $result = array();
        if($customQuery) {
            $result = InstanceProvider::getRepository()->getCpCallLocal()->findAllCalls($customQuery,$orderBy, $page, $limit);

            //foreach($result as &$retorno){
                //for($i=1; $i<6;$i++){
                    //$retorno["DATE_" .$i] = $retorno["DATE_". $i] ? $retorno["DATE_". $i] : "";
                //}
            //}
        }
        return $result;
    }

    public function getQueryAllCall($conditions, $situationFilters, $clientePmFilter, $clienteIntranetFilter,
                                    $clienteCRMFilter, $filterSLA = '',$filterSLAClas = '',$filterSLAAss = '', $cdProd = '')
    {
        list($callType, $whereFiltersProductCalls, $situationProductCalls, $specialFiltersProductCalls) = $this->buildWhereFilterToProductCalls($conditions, $situationFilters);

        $mobileProd = $soliProd = [];
        $queryPmCalls = $queryIntranetCalls = $queryCrmCalls = "";

        if(is_array($cdProd)) {
            foreach($cdProd as $cdproduc) {
                if(strlen($cdproduc) !== 3) {
                    array_push($mobileProd, $cdproduc);
                } else {
                    array_push($soliProd, $cdproduc);
                }
            }
        }
        $soliFilter = '';
        if($soliProd) {
            $soliFilter = "AND PRO.CDARVPROD LIKE '%";
            foreach($soliProd as $key => $strProd) {
                if($key > 0) {
                    $soliFilter .= "OR PRO.CDARVPROD LIKE '%";
                }
                $soliFilter .= $strProd . "' ";
            }
        } else {
            $callType = Entities\CpCall::TYPE_ISSUE;
        }
        $mobFilter = '';
        if($mobileProd){
            $mobFilter = "AND PRODUCT IN (";
            foreach($mobileProd as $key => $strProd){
                if($key!=0){
                    $mobFilter .= "," . $strProd;
                }else{
                  $mobFilter .= $strProd  ;
                }
                $mobFilter .= ")";
            }
        } else {
            if($callType) {
                $callType = '';
            } else {
                $callType = Entities\CpCall::TYPE_RICH_CLIENT;
            }
        }
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        if ($callType == Entities\CpCall::TYPE_ISSUE) {
            $queryPmCalls = $cpCallRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter,$filterSLA,$filterSLAClas, $mobFilter);
        } else if ($callType == Entities\CpCall::TYPE_RICH_CLIENT) {
            $queryIntranetCalls = $cpCallRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter, $filterSLA,$filterSLAAss, $soliFilter);
        } else {
            $queryPmCalls = $cpCallRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter, $filterSLA,$filterSLAClas, $mobFilter);
            $queryIntranetCalls = $cpCallRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls,
                $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter,$filterSLA,$filterSLAAss, $soliFilter);
        }

        $customQuery = $queryPmCalls;

        if ($queryIntranetCalls) {
            $customQuery .= (empty($customQuery) ? "" : " UNION ") . $queryIntranetCalls;
        }

        if ($queryCrmCalls) {
            $customQuery .= (empty($customQuery) ? "" : " UNION ") . $queryCrmCalls;
        }
        return $customQuery;
    }

    private function buildWhereFilterToProductCalls($conditions, $allSituationDividedByType)
    {
        $specialFilters = '';
        $callType = '';
        $situation = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        foreach($conditions as $key => $filter) {
            if($filter['columnName'] == 'PRODUCT_TYPE') {
                $specialFilters .= " AND EXISTS(SELECT * FROM SOLISERV SOLIC WHERE SOLIC.CDPRODGRUP = ";
                $specialFilters .= "'". $filter['value'] . "' AND SOLIC.NRSOLISERV = C.CALL_RELATIONSHIP)";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'CALL_WITHOUT_LABELS_CHECK') {
                if($filter['value'] == '%Y%') {
                    $userId = $this->environment->getCurrentUserId();
                    $specialFilters .= " AND NOT EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN (SELECT NRCALLLABEL FROM CP_CALL_LABEL WHERE CDOPERINCLUSAO = '$userId') OR CLR.LABEL IS NULL)";
                }
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'CALL_LABELS') {
                $conditions[$key]["columnName"] = "CLR.LABEL";
                $specialFilters .= "AND EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN(".implode(',',$filter['value'])."))";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'TYPE') {
                $callType = $conditions[$key]['value'];
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'STATUS') {
                $situation = $this->situationFilter($conditions[$key]['value'], $allSituationDividedByType);
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'NRCALL'){
                $conditions[$key]['columnName'] = 'ID';
            } else if($filter['columnName'] == 'CLASSIFICATION_NAME'){
                $conditions[$key]['columnName'] = 'CLASSIFICATION';
            }
        }
        $conditionsQuery = DevOpsUtilities::createQueryFiltered($conditions);

        return array($callType, $conditionsQuery, $situation, $specialFilters);
    }

    public function situationFilter($situationTypes, $allSituationDividedByType, $aliasContato = 'CONT', $aliasTaskStattus = 'TS', $aliasSoliserv = 'SER')
    {
        $situationFilter = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        if(!empty($situationTypes)){
            $issue = array();
            $contato = array();
            $soliserv  = array();
            asort($situationTypes);
            $isWaitingIssue= '';
            foreach ($situationTypes as $situationType) {
                if ($situationType == Entities\CpCall::CLOSE_SITUATION) {
                    $issue = $allSituationDividedByType['PM']['CLOSED'];
                    $contato = $allSituationDividedByType['CRM']['CLOSED'];
                    $soliserv[] = $allSituationDividedByType['GEP']['CLOSED'];
                } else if ($situationType == Entities\CpCall::OPEN_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['OPENED']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['OPENED']);
                    $soliserv[] = $allSituationDividedByType['GEP']['OPENED'];
                } else  if ($situationType == Entities\CpCall::PROGRESS_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['IN_PROGRESS']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['IN_PROGRESS']);
                    $soliserv[] = $allSituationDividedByType['GEP']['IN_PROGRESS'];
                } else if ($situationType == Entities\CpCall::WAITING_SPONSOR_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['WAITING']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['WAITING']);
                    $soliserv[] = $allSituationDividedByType['GEP']['WAITING'];
                    $isWaitingIssue = true;
                }
            }
            $question = <<<SQL
                         'Q' = (SELECT NVL(CONTROL,'I') FROM (SELECT CTY.CONTROL
                                                FROM COMMENTS COM
                                                LEFT JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                WHERE COM.TASK = TAS.ID
                                                ORDER BY COM.ID DESC)
                                                WHERE ROWNUM = 1 )
SQL;

            if(!empty($issue)){
                $contatoString = implode($contato, ',');
                $issueString = implode($issue, ',');
                $soliservString = implode($soliserv, 'OR');
                $situationFilter['CONTATO'] = " AND $aliasContato.NRSEQSITUACAO IN($contatoString)";
                $situationFilter['SOLISERV'] = " AND ($soliservString)";

                if($isWaitingIssue){
                    $situationFilter['ISSUE'] = " AND ($aliasTaskStattus.ID IN($issueString) OR $question )";
                }else{
                    $situationFilter['SOLISERV'] .= "AND NOT ".$allSituationDividedByType['GEP']['WAITING'];
                    $situationFilter['ISSUE'] = " AND $aliasTaskStattus.ID IN($issueString)"." AND NOT ".$question;
                }
            }
        }
        return $situationFilter;
    }

    public function getAllSituationDividedByType($aliasSoliserv = 'SER')
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $waitingTicket    = Utilities::getWaitingTicket();
        $processingType   = Utilities::getExcludedWaitingCustomerLogFaseTram();
        $allSituation     = $cpCallRepository->getAllSituation($this->environment->getCurrentOrganizationId(), $waitingTicket);
        //$cdcliente        = $this->environment->getCurrentCdCliente();
        $cdcliente = 'J02905110000128';
        $waitingGEP       = " (EXISTS (SELECT CDOPERADOR FROM OPERADOR WHERE CDCLIENTE = '$cdcliente' AND CDOPERADOR = $aliasSoliserv.CDOPERRESP AND $aliasSoliserv.CDFASETRAM NOT IN ('$processingType')" ;
        list($openedGep, $closedGep) = $this->getBeginAndEndCdfasetram();
        $allSituationDividedByType = array (
            'GEP' => [
                'CLOSED' => " $aliasSoliserv.CDFASETRAM = $closedGep OR ($aliasSoliserv.IDSOLIENCE = 'S')",
                'OPENED' => " $aliasSoliserv.CDFASETRAM = $openedGep AND ($aliasSoliserv.IDSOLIENCE <> 'S')",
                'WAITING' => $waitingGEP." AND $aliasSoliserv.CDFASETRAM NOT IN ($closedGep) AND ($aliasSoliserv.IDSOLIENCE <> 'S'))) ",
                'IN_PROGRESS' => " $aliasSoliserv.CDFASETRAM NOT IN ($openedGep,$closedGep) AND ($aliasSoliserv.IDSOLIENCE <> 'S') ",
            ],
            'PM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
            'CRM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
        );
        foreach ($allSituation as $situation) {
            $allSituationDividedByType[$situation['PRODUCT']][$situation['SITUATION']][] = $situation['ID'];
        }
        return $allSituationDividedByType;
    }

    public function getBeginAndEndCdfasetram()
    {
        $paramgeral = InstanceProvider::getRepository()->getCpCall()->getBeginAndEndCdfasetram($this->environment->getCurrentOrganizationId());
        return array($paramgeral['CDFATRAMIWEB'], $paramgeral['CDFATRAMENCE']);
    }

    public function findCallComments($callRelationship, $callType)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $this->callRepository->findOneBy(array('callRelationship' => $callRelationship));
        $commentsRepository = InstanceProvider::getRepository()->getCommentsLocal();

        //$ticket = empty($productCall) ? null : $productCall->getTicketRelationship();
        $userId = $this->environment->getCurrentUserId();
        $comments = [];

        if($callType == Entities\CpCall::TYPE_ISSUE) {
            $comments = $commentsRepository->getCommentsOfTaskExceptCommentedBy($callRelationship);
            $attachments = InstanceProvider::getRepository()->getAttachmentsLocal()->getTaskAttachmentsExceptUploadedBy($callRelationship);
            foreach ($comments['data'] as &$comment) {
                $comment['ATTACHMENT'] = array();
                foreach ($attachments['data'] as $attachment) {
                    if($attachment['COMMENT_ID'] == $comment['ID']){
                        $comment['ATTACHMENT'][] = $attachment;
                    }
                }
                $comment['NOT_EXIST_FILE'] = !count($comment['ATTACHMENT'] );
            }
        } else if($callType == Entities\CpCall::TYPE_RICH_CLIENT) {
            $situationFilters = $this->getAllSituationDividedByType();
            $productCall = $this->callRepository->findSoliservById($callRelationship, $situationFilters, $userId);
            if(!empty($productCall)){
                $comments = $commentsRepository->getProductCallCommentsFromSolicitacao($callRelationship);
                $attachments = InstanceProvider::getService()->getINTRANETIntegration()->getAttachments(null,
                    $callRelationship);
                foreach ($comments['data']  as &$comment) {
                    $comment['ATTACHMENT'] = array();
                    foreach ($attachments as $attachment) {
                        if($attachment['FOLDER'] === $comment['ORDERBY'] ){
                            $comment['ATTACHMENT'][] = $attachment;
                        }
                    }
                    $comment['NOT_EXIST_FILE'] = !count($comment['ATTACHMENT'] );
                }
            }
        }
        return $comments;
    }

    public function findCallAttachments($callRelationship, $callType, $commentId, $filename)
    {
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $this->callRepository->findOneBy(array('callRelationship' => $callRelationship));
        $callId = empty($productCall) ? null : $productCall->getNrcall();

        if($callType == Entities\CpCall::TYPE_ISSUE) {
            $commentIdQuery = "";
            if($commentId) {
                $commentIdQuery = "AND C.ID = " . $commentId;
            }
            $attachments = InstanceProvider::getRepository()->getAttachmentsLocal()->getTaskAttachmentsExceptUploadedBy($callRelationship, $commentIdQuery);
            foreach($attachments['data'] as $index => $attachment) {
                $attachments[$index]['__fromIssue'] = true;
            }
        } else {
            $result = InstanceProvider::getService()->getINTRANETIntegration()->getAttachments($callId,
                $callRelationship, $filename);
            $attachments = array('data'=> $result,'pagination' => array('limit'=> 999, 'page' => 1, 'total' => count($result)));
        }
        return  $attachments;
    }

    public function getCall( $callRelationship, $callType)
    {
        $call = array();
        if($callType == self::TYPE_RICH_CLIENT || $callType == self::TYPE_ISSUE){
            $call = $this->findProductCall($callRelationship, $callType);
        }

        return $call;
    }

    public function findCallTypeIssue($id)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $situationFilters = $this->getAllSituationDividedByType();
        return $cpCallRepository->findCallTypeIssue($id,$situationFilters);
    }

    public function findCallTypeRichClient($id)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $userId = $this->environment->getCurrentUserId();
        $situationFilters = $this->getAllSituationDividedByType();
        return $cpCallRepository->findCallTypeRichClient($id,$userId,$situationFilters);
    }

    public function getCallTypeDescription($type) {
        switch($type) {
            case self::TYPE_ISSUE:
                $desc = Entities\CpCall::TYPE_ISSUE_DESC;
                break;
            case self::TYPE_RICH_CLIENT:
                $desc = Entities\CpCall::TYPE_RICH_CLIENT_DESC;
                break;
            default:
                $desc = '';
        }
        return $desc;
    }

    private function findProductCall($callRelationship, $callType)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCallLocal();
        $soliservRepository = InstanceProvider::getRepository()->getSoliservLocal();
        $userId             = $this->environment->getCurrentUserId();
        $productCall        = array();

        $situationFilters = $this->getAllSituationDividedByType();
        /** @var Entities\CpCall $call */
        $call = InstanceProvider::getRepository()->getCpCall()->findOneBy(array('callRelationship' => $callRelationship));
        if(!empty($call)){
            if($call->getType() == \Service\Call::TYPE_ISSUE) {
                $productCall = $this->findCallTypeIssue($call->getNrcall());
                $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($call->getType());
                if(empty($productCall['CALL_LABELS'])){
                    $productCall['CALL_LABELS'] = array();
                } else {
                    $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
                }

            } else if ($call->getType() == \Service\Call::TYPE_RICH_CLIENT) {
                $productCall = $this->findCallTypeRichClient($call->getNrcall());
                $productCall['DESCRIPTION'] = Utilities::rtf2textPlain($productCall['DESCRIPTION'], true);
                $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($call->getType());
                if(empty($productCall['CALL_LABELS'])){
                    $productCall['CALL_LABELS'] = array();
                } else {
                    $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
                }

                $soliServ = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $productCall['CALL_RELATIONSHIP']));

                $useLicense = $this->getLicencausoBySoliserv($soliServ);
                if($useLicense){
                    $productCall['USE_LICENSE'] = $useLicense->getNrlicencauso();
                    $productCall['USE_LICENSE_LABEL'] = $productCall['USE_LICENSE'] . ' - ' . $useLicense->getNmfantluso();
                }else{
                    $productCall['USE_LICENSE'] = '';
                    $productCall['USE_LICENSE_LABEL'] = '';
                }
            }
        } else if ($callType == "I"){
            $productCall = $cpCallRepository->findIssueById($callRelationship, $situationFilters);
            $productCall['CALL_RELATIONSHIP'] = $productCall['ID'];
            $productCall['ID'] = null;
            $productCall['TYPE'] = $callType;
            $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($callType);
            $productCall['CALL_LABELS'] = empty($productCall['CALL_LABELS']) ? array() : explode(',', $productCall['CALL_LABELS']);
        } else if ($callType == "S"){
            $productCall = $cpCallRepository->findSoliservById($callRelationship, $situationFilters, $userId);
            $productCall['DESCRIPTION'] = Utilities::rtf2textPlain($productCall['DESCRIPTION'], true);
            $productCall['CALL_RELATIONSHIP'] = $productCall['ID'];
            $productCall['TYPE'] = $callType;
            $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($callType);
            $productCall['ID'] = null;


            $soliServ = InstanceProvider::getRepository()->getSoliserv()->findOneBy(array('nrsoliserv' => $productCall['CALL_RELATIONSHIP']));
            $useLicense = $this->getLicencausoBySoliserv($soliServ);

            $productCall['USE_LICENSE'] = $useLicense ? $useLicense->getNrlicencauso() : '';
            $productCall['USE_LICENSE_LABEL'] = $useLicense ? $productCall['USE_LICENSE'] . ' - ' . $useLicense->getNmfantluso() : '';

        }

        if(!empty($productCall)){
            $productCall['MAIN_TYPE_CALL'] = self::MAIN_TYPE_OF_PRODUCT_CALL;
        }

        return $productCall;
    }

    /**
     * @param \Teknisa\Model\Teknisa\Entities\Soliserv  $soliserv
     * @return null |\Teknisa\Model\Teknisa\Entities\Licencauso
     */
    private function getLicencausoBySoliserv(\Teknisa\Model\Teknisa\Entities\Soliserv $soliserv = null)
    {
        $licencauso = null;
        if ($soliserv->getCdsetor() != null) {
            $id = array(
                "cdcliente" => $soliserv->getCdcliente(),
                "nrlicencauso" => $soliserv->getNrlicencauso(),
                "nrorg" => $soliserv->getNrorg(),
            );
            $licencauso = InstanceProvider::getRepository()->getLicencauso()->findOneBy($id);
        }
        return $licencauso;
    }

}
